#!/usr/bin/perl -w

# $Id: check_http.pl,v 1.6 2006/01/29 04:25:56 knitterb Exp $

## SSL!!!!
# for ssl usage, and client certs, see:
#     http://search.cpan.org/~chamas/Crypt-SSLeay/

use HTTP::Request;
use LWP::UserAgent;
use Data::Dumper;
use Getopt::Long;
use Date::Manip;
use Time::HiRes qw(usleep ualarm gettimeofday tv_interval);

my @ISA=qw(LWP::UserAgent);
sub redirect_ok {
	return 1;
}

use strict;

my $DEBUG=undef;
my %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3, 'DEPENDENT'=>4);

my ($url,$post,$crit,$warn,$authuser,$authpass,@params,@proxies,
	$help,$output,$regex,$sha1,$sha1set,$rc,$dtd,$postfile,$authrealm,$display,$header,
	@env_vars, $last_modified);
my $type;
my $code=200;

GetOptions (
	# REQUIRED
	'url|l=s'=>\$url

	#optional
	,'display=s'=>\$display
	,'help|h|?'=>\$help
	,'crit|c=i'=>\$crit
	,'warn|w=i'=>\$warn
	,'user|u=s'=>\$authuser
	,'pass|p=s'=>\$authpass
	,'realm|m=s'=>\$authrealm
	,'post|x=s'=>\$post
	,'postfile|f=s'=>\$postfile
	,'type|t=s'=>\$type
	,'output|o'=>\$output
	,'regex|r=s'=>\$regex
	,'sha1|S'=>\$sha1
	,'sha1set'=>\$sha1set
	,'dtd=s'=>\$dtd
	,'debug|d'=>\$DEBUG
    ,'param|a=s'=>\@params
	,'proxy=s'=>\@proxies
	,'code=i'=>\$code
	,'header=s'=>\$header
	,'last_modified=s'=>\$last_modified
	,'env=s'=>\@env_vars
);

if (!defined($DEBUG)) {
	$DEBUG=0;
} else {
	$DEBUG=1;
}

if (defined($help)) {
	print_usage(undef);
	exit($ERRORS{"UNKNOWN"});
}

my $check_res=check_vars();
if (defined($check_res)) {
	print_usage($check_res);
	print "UNKNOWN - ".$check_res."\n";
	exit($ERRORS{"UNKNOWN"});
}

# set up extra env vars
if (@env_vars>0) {
	print "ENV Vars: ".Dumper(\@env_vars)."\n" if $DEBUG;
	for my $i (@env_vars) {
		my ($name, $value)=split(/=/, $i);
		$ENV{$name}=$value;
	}
}

my $ua=new LWP::UserAgent;
$ua->agent("Nagios/check_http.pl");

# TODO make this use more than just the first one
if (@proxies>0) {
	print "Using a proxy ".$proxies[0]."\n" if $DEBUG;
	$ua->proxy('http', $proxies[0]);
}

if (defined($authuser) && defined($authpass)) {
	# add this to our $browser 's "key ring"
	print "Adding credentials $authuser:$authpass for $authrealm\n" if $DEBUG;
	my $uri=URI->new($url);
	$ua->credentials($uri->host_port(), $authrealm, $authuser, $authpass);
}

my $req_header=new HTTP::Headers;

# let's make sure nothing is every cached
$req_header->header('Pragma' => 'no-cache');
$req_header->header('Cache-Control' => 'no-cache');

my $req;
my $send_doc=undef;

if (defined($postfile)) {
	print "Found a postfile\n" if $DEBUG;
	$post="";
	open (POSTFILE, "<$postfile");
	while (<POSTFILE>) {
		$post.=$_;
	}
	close(POSTFILE);
}

if (defined($post)) {
	if (@params>0) {
		for (@params) {
			my ($key, $val)=split(/=/, $_, 2);
			$post=~s/$key/$val/g;
		}
	}
	print "posting: $post\n" if $DEBUG;

	$req_header->header('Content-Type' => $type);
	$req=new HTTP::Request ("POST", $url, $req_header, $post);
	push @{ $ua->requests_redirectable }, 'POST';
} else {
	$req=new HTTP::Request ("GET", $url, $req_header);
} 

print "Redir: ".Dumper($ua->requests_redirectable) if $DEBUG;

# send off the request
my $start_time=[gettimeofday];
print "Req: ".Dumper($req)."\n" if $DEBUG;

my $res=$ua->request($req);

# get our duration in ms
my $duration=tv_interval($start_time)*1000;

# get rid of that xml doc if it exists
if (defined($send_doc)) {
	$send_doc->dispose();
}

my $content="";

# now check for the results to see if they are good
print "Res: ".Dumper($res)."\n" if $DEBUG;
if ($res->is_success() || $res->code()==$code) {
	# request was a successful response
	if (defined($output)) {
		$content=$res->content();
	}
} else {
	print "CRITICAL - Error Fetching $url [".
		$res->code()."] ".$res->message()." $content\n";
	exit($ERRORS{"CRITICAL"});
}

# so we have a good response and a good document at this point, now check
# for timing to see if that is within the range required

if (defined($crit) && ($crit*1000)<$duration) {
	output("CRITICAL - document delayed", $content,
		$duration, $ERRORS{"CRITICAL"});
}
if (defined($warn) && ($warn*1000)<$duration) {
	output("WARNING - document delayed", $content,
		$duration, $ERRORS{"WARNING"});
}

if (validate_content($res->content())!=0) {
	output("CRITICAL - document mismatch", $content,
		$duration, $ERRORS{"CRITICAL"});
}

$rc=validate_sha1($res->content());
if($rc==2) {
	output("WARNING - checksum not found", $content,
		$duration, $ERRORS{"WARNING"});
}elsif($rc!=0) {
	output("CRITICAL - document mismatch", $content,
		$duration, $ERRORS{"CRITICAL"});
}

if(defined($header)) {
	my ($key, $value)=split(/=/, $header, 2);
	my $res_headers=$res->headers();
	print Dumper($res_headers) if $DEBUG;
	my $h=$res_headers->header($key);
	if (!defined($h)) {
		output("WARNING - Missing header field ``$key''", $content,
			$duration, $ERRORS{"WARNING"});
	}
	print "Looking up $value\n" if $DEBUG;
	if ($h!~m/$value/i) {
		output("CRITICAL - Mismatch header field ``$key'' for $header",$content,
			$duration, $ERRORS{"CRITICAL"});
	}
}

if(defined($last_modified)) {
	print "Last Modified: $last_modified\n" if $DEBUG;
	my $res_headers=$res->headers();
	print Dumper($res_headers) if $DEBUG;
	my $s_date=UnixDate(ParseDate($res_headers->header('last_modified')),"%s");
	my $c_date=UnixDate("today", "%s");
	my $delta =$c_date-$s_date;
	if ($delta>$last_modified) {
		output("CRITICAL - File not modified within $last_modified seconds".
			" ($delta)", $content, $duration, $ERRORS{"CRITICAL"});
	}
	$content="modified ${delta}s";
}

output("OK - document successful", $content, $duration, $ERRORS{"OK"});

# we should not get here
print "UNKNOWN - Problem with the script\n";
exit($ERRORS{"UNKNOWN"});

sub output {
	my ($text, $content, $time, $code)=@_;
	if ($display eq "cricket") {
		print "$time\n";
		exit($code);
	} else {
		# default or nagios
		print "$text [".$time." ms] $content\n";
		exit($code);
	}
}

sub validate_content {
	my ($content)=@_;
	if (defined($regex)) {
		print "Checking content for ``$regex'' in ".length($content)
			." characters\n" if $DEBUG;
		if ($content!~m/$regex/i) {
			print "Content mismatch on ``$regex''\n" if $DEBUG;
			return(1);
		}
	}
	return(0);
}

use Digest::SHA1  qw(sha1 sha1_hex sha1_base64);
use GDBM_File ;
sub validate_sha1 {
	my ($content)=@_;
	my ($digest, %hash, $return_val);
	$return_val=0;
	if (defined($sha1)) {
        	$digest = sha1_base64($content);
		tie %hash, 'GDBM_File', "/var/tmp/sites_sha1.dbm", &GDBM_WRCREAT, 0640;
		# Use the %hash array.
		if(!defined($sha1set) and $hash{$url}) {
			if($hash{$url} ne $digest ) {
				print "'$hash{$url}' !~ '$digest'\n" if $DEBUG;
				$return_val=1;
			}
		} else {
			$hash{$url}=$digest;
			$return_val=2;
		}
		untie %hash ;
	}
	return($return_val);
}

sub print_usage {
        my ($msg)=@_;
        if ($msg) {
                print "\n";
                print "$msg\n";
        }
        print "\n";
        print "This will check the url for a valid HTML document response.\n";
        print "Additional checks can be specified below.\n";
        print "\n";
        print "REQUIRED PARAMETERS:\n";
		print "--url -l\tThe url to post the request to\n";
		print "\n";
        print "OPTIONAL PARAMETERS:\n";
		print "--display\tThe display mode: nagios (default) or cricket\n";
		print "--user -u\tThe user for authentication (requires password)\n";
		print "  --pass -p\tThe pasword for authentication (requires user)\n";
		print "  --realm -m\tThe realm for authentication (requires user & pass)\n";
		print "--post -x\tThe data to post\n";
		print "--postfile -d\tThe data to post - from a file\n";
		print "  --type -t\tContent-type of post (default: text/plain)\n";
		print "--output -o\tprint the output on the first line of stdout\n";
		print "--help -h -?\tThis help message\n";
		print "--debug -d\tTurn on debug mode\n";
		print "\n";
        print "OPTIONAL CHECKS (executed in order below):\n";
		print "--crit -c\tThe number of request seconds for critical state\n";
		print "--warn -w\tThe number of request seconds for critical state\n";
		#print "--dtd\tCheck the returned document against a dtd\n";
		print "--regex -r\tDo a case insensitive regex on the output (m//i)\n";
		print "--sha1 -S\tDo a checksum of the webpage using a SHA1 value stored in a dbm\n";
		print "--sha1set\tReset the sha1 value in the dbm file, must use --sha1 as well\n";
		print "--param -a\ta key value pair for match replace (multiple)\n";
		print "--proxy \ta proxy server to use (multiple)\n";
		print "--code\t\tThe expected HTTP response code (default 200)\n";
		print "--env\t\tENV vars to set (i.e.  HTTPS_PKCS12_FILE=file.p12)\n";
		print "--last_modified\tCheck that the 'Last Modified' header\n";
		print "\t\tstates this was modified 'x' seconds ago\n";
		print "\n";
}

sub check_vars {
	if (!defined($url)) {
		return "Missing url";
	}
	if (defined($authuser) || defined($authpass)) {
		if (!defined($authuser)|| !defined($authpass)) {
			return("Missing user or pass paramater (pair required)");
		}
	}
	if (defined($crit) && defined($warn)) {
		if ($crit<$warn) {
			return("Critical setting must be >= warning setting");
		}
	}
	if (defined($type)) {
		if (!defined($post) && !defined($postfile)) {
			return("Post/postfile parameter missing when type parameter found");
		}
	}
	if (!defined($display)) {
		$display="nagios";
	}
	return undef;
}


